@@ -3,7 +3,7 @@ import gtk
 from kiwi.ui.views import BaseView, quit_if_last
 
 class Person:
-    gender = 'Male'
+    gender = "Male"
 
 class GenderSelection(BaseView):
     gladefile = 'gender'
