@@ -20,6 +20,7 @@
 #
 # Author(s): Lorenzo Gil Sanchez <lgs@sicem.biz>
 #            Johan Dahlin <jdahlin@async.com.br>
+#            Ronaldo Maia <romaia@async.com.br>
 #
 
 """Data type converters with locale and currency support.
@@ -44,6 +45,8 @@ import sys
 import time
 
 from kiwi import ValueUnset
+from kiwi.enums import Alignment
+from kiwi.python import enum
 
 try:
     from decimal import Decimal, InvalidOperation
@@ -108,6 +111,14 @@ class ConverterRegistry:
         del self._converters[ctype]
 
     def get_converter(self, converter_type):
+        # This is a hack:
+        # If we're a subclass of enum, create a dynamic subclass on the
+        # fly and register it, it's necessary for enum.from_string to work.
+        if (issubclass(converter_type, enum) and
+            not converter_type in self._converters):
+            self.add(type(enum.__class__.__name__ + 'EnumConverter',
+                       (_EnumConverter,), dict(type=converter_type)))
+
         try:
             return self._converters[converter_type]
         except KeyError:
@@ -116,7 +127,7 @@ class ConverterRegistry:
     def get_converters(self, base_classes=None):
         if base_classes is None:
             return self._converters.values()
-        
+
         converters = []
         if object in base_classes:
             #: Ugly, but cannot remove from tuple!
@@ -189,18 +200,24 @@ class BaseConverter(object):
     Abstract converter used by all datatypes
     @cvar type:
     @cvar name: The name of the datatype.
+    @cvar align: The alignment of the datatype. Normally right for numbers
+                 and dates, left for others. Default is left.
     """
     type = None
     name = None
+    align = Alignment.LEFT
 
     def get_compare_function(self):
         """
-        @returns:
+        This can be overriden by a subclass to provide a custom comparison
+        function.
+        @returns: cmp
         """
         return cmp
 
     def as_string(self, value, format):
         """
+        Convert the value to a string using the specificed format.
         @param value:
         @param format:
         @returns:
@@ -208,13 +225,15 @@ class BaseConverter(object):
 
     def from_string(self, value):
         """
+        Convert a value from a string.
         @param value:
         @returns:
         """
 
     def get_mask(self):
         """
-        @returns:
+        Returns the mask of the entry or None if not specified.
+        @returns: the mask or None.
         """
         return None
 
@@ -249,6 +268,7 @@ converter.add(_UnicodeConverter)
 class _IntConverter(BaseConverter):
     type = int
     name = _('Integer')
+    align = Alignment.RIGHT
 
     def as_string(self, value, format=None):
         """Convert a float to a string"""
@@ -308,6 +328,7 @@ converter.add(_BoolConverter)
 class _FloatConverter(BaseConverter):
     type = float
     name = _('Float')
+    align = Alignment.RIGHT
 
     def as_string(self, value, format=None):
         """Convert a float to a string"""
@@ -359,6 +380,7 @@ converter.add(_FloatConverter)
 class _DecimalConverter(_FloatConverter):
     type = Decimal
     name = _('Decimal')
+    align = Alignment.RIGHT
     def from_string(self, value):
         if value == '':
             return ValueUnset
@@ -390,17 +412,23 @@ DATE_REPLACEMENTS_WIN32 = [
     (re.compile('yy?'), '%y')
     ]
 
+# This "table" contains, associated with each
+# key (ie. strftime "conversion specifications")
+# a tuple, which holds in the first position the
+# mask characters and in the second position
+# a "human-readable" format, used for outputting user
+# messages (see method _BaseDateTimeConverter.convert_format) 
 DATE_MASK_TABLE = {
-    '%m': '00',
-    '%y': '00',
-    '%d': '00',
-    '%Y': '0000',
-    '%H': '00',
-    '%M': '00',
-    '%S': '00',
-    '%T': '00:00:00',
+    '%m': ('00', _('mm')),
+    '%y': ('00', _('yy')),
+    '%d': ('00', _('dd')),
+    '%Y': ('0000', _('yyyy')),
+    '%H': ('00', _('hh')),
+    '%M': ('00', _('mm')),
+    '%S': ('00', _('ss')),
+    '%T': ('00:00:00', _('hh:mm:ss')),
     # FIXME: locale specific
-    '%r': '00:00:00 LL',
+    '%r': ('00:00:00 LL', _('hh:mm:ss LL')),
     }
 
 class _BaseDateTimeConverter(BaseConverter):
@@ -410,6 +438,7 @@ class _BaseDateTimeConverter(BaseConverter):
     @cvar lang_constant:
     """
     date_format = None
+    align = Alignment.RIGHT
 
     def __init__(self):
         self._keep_am_pm = False
@@ -479,7 +508,7 @@ class _BaseDateTimeConverter(BaseConverter):
     def get_mask(self):
         mask = self.get_format()
         for format_char, mask_char in DATE_MASK_TABLE.items():
-            mask = mask.replace(format_char, mask_char)
+            mask = mask.replace(format_char, mask_char[0])
 
         return mask
 
@@ -491,8 +520,20 @@ class _BaseDateTimeConverter(BaseConverter):
         if value is None:
             return ''
 
+        if isinstance(value, (datetime.date, datetime.datetime)):
+            if value.year < 1900:
+                raise ValidationError(
+                    _("You cannot enter a year before 1900"))
+
         return value.strftime(format)
 
+    def _convert_format(self, format):
+        "Convert the format string to a 'human-readable' format"
+        for char in DATE_MASK_TABLE.keys():
+            format = format.replace(char, DATE_MASK_TABLE[char][1])
+
+        return format
+
     def from_string(self, value):
         "Convert a string to a date"
 
@@ -508,11 +549,18 @@ class _BaseDateTimeConverter(BaseConverter):
             # time.strptime (python 2.4) does not support %r
             # pending SF bug #1396946
             dateinfo = time.strptime(value, format)
-            return self.from_dateinfo(dateinfo)
+            date = self.from_dateinfo(dateinfo)
         except ValueError:
             raise ValidationError(
                 _('This field requires a date of the format "%s" and '
-                  'not "%s"') % (format, value))
+                  'not "%s"') % (self._convert_format(format), value))
+
+        if isinstance(date, (datetime.date, datetime.datetime)):
+            if date.year < 1900:
+                raise ValidationError(
+                    _("You cannot enter a year before 1900"))
+        return date
+
 
 class _TimeConverter(_BaseDateTimeConverter):
     type = datetime.time
@@ -567,6 +615,26 @@ class _ObjectConverter(BaseConverter):
     from_string = None
 converter.add(_ObjectConverter)
 
+class _EnumConverter(BaseConverter):
+    type = enum
+    name = _('Enum')
+
+    def as_string(self, value, format=None):
+        if not isinstance(value, self.type):
+            raise ValidationError(
+                "value must be an instance of %s, not %r" % (
+                self.type, value))
+        return value.name
+
+    def from_string(self, value):
+        names = self.type.names
+        if not value in names:
+            raise ValidationError(
+                "Invalid value %s for enum %s" % (value, self.type))
+        return names[value]
+
+converter.add(_EnumConverter)
+
 def lformat(format, value):
     """Like locale.format but with grouping enabled"""
     return locale.format(format, value, 1)
