@@ -1,7 +1,7 @@
 #
 # Kiwi: a Framework and Enhanced Widgets for Python
 #
-# Copyright (C) 2006 Async Open Source
+# Copyright (C) 2006-2007 Async Open Source
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -38,3 +38,31 @@ class Alignment(enum):
 
 class Direction(enum):
     (LEFT, RIGHT) = (1, -1)
+
+class ListType(enum):
+    """
+    - NORMAL: Add, Remove, Edit
+    - REMOVEOLY: Remove
+    - UNADDABLE: Remove, Edit
+    - UNEDITABLE: Add, Remove
+    - UNREMOVABLE: Add, Edit
+    - READONLY: No buttons
+    """
+    (NORMAL,
+     READONLY,
+     REMOVEONLY,
+     UNREMOVABLE,
+     UNADDABLE,
+     UNEDITABLE) = range(6)
+
+class SearchFilterPosition(enum):
+    """
+    An enum used to indicate where a search filter should be added to
+    a SearchContainer::
+
+      - TOP: top left corner
+      - BOTTOM: bottom
+    """
+    (TOP,
+     BOTTOM) = range(2)
+
