@@ -96,7 +96,9 @@ class Environment:
         if not os.path.isdir(path):
             raise EnvironmentError("path %s must be a directory" % path)
 
-        self._resources.setdefault(resource, []).append(path)
+        reslist = self._resources.setdefault(resource, [])
+        if not path in reslist:
+            reslist.append(path)
 
     def add_resources(self, **kwargs):
         for resource, path in kwargs.items():
@@ -105,7 +107,7 @@ class Environment:
                     self.add_resource(resource, path)
 	        except EnvironmentError:
                     continue
-            
+
             self.add_resource(resource, path)
 
     def find_resource(self, resource, name):
@@ -212,8 +214,10 @@ class Library(Environment):
             root = os.path.abspath(os.path.join(dirname, root))
         Environment.__init__(self, root=root)
 
-        sys.path.insert(0, os.path.join(root, 'lib', 'python%d.%d' %
-                                        sys.version_info[:2], 'site-packages'))
+        basedir = os.path.join(root, 'lib', 'python%d.%d' %
+                               sys.version_info[:2], 'site-packages')
+        if os.path.exists(basedir):
+            sys.path.insert(0, basedir)
         g = globals()
         l = locals()
         try:
@@ -236,6 +240,7 @@ class Library(Environment):
                                  "global_resources attribute" % name)
             self.add_resources(**module.resources)
             self.add_global_resources(**module.global_resources)
+            self.prefix = module.prefix
 
         self.uninstalled = uninstalled
 
@@ -290,6 +295,9 @@ class Library(Environment):
             return
         directory = gettext.bindtextdomain(domain, localedir[0])
         self._check_translation(domain, directory)
+        # For libglade, but only on non-win32 systems
+        if hasattr(locale, 'bindtextdomain'):
+            locale.bindtextdomain(domain, localedir[0])
 
         # Gtk+ only supports utf-8, it makes no sense to support
         # other encodings in kiwi it self
@@ -303,6 +311,9 @@ class Library(Environment):
         @param domain: the domain
         """
         gettext.textdomain(domain)
+        # For libglade, but only on non-win32 systems
+        if hasattr(locale, 'textdomain'):
+            locale.textdomain(domain)
 
     def add_global_resource(self, resource, path):
         """Convenience method to add a global resource.
