@@ -91,6 +91,10 @@ An enchanced version of GtkEntry that supports icons and masks
 
 import gettext
 import string
+try:
+    set
+except NameError:
+    from sets import Set as set
 
 import gobject
 import pango
@@ -329,6 +333,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
 
     def get_mask(self):
         """
+        Get the mask.
         @returns: the mask
         """
         return self._mask
@@ -634,11 +639,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
             return completion
 
         if old and isinstance(old, KiwiEntryCompletion):
-            if old.completion_timeout:
-                gobject.source_remove(old.completion_timeout)
-                old.completion_timeout = 0
-
-            old._disconnect_completion_signals()
+            old.disconnect_completion_signals()
 
         self._completion = completion
 
@@ -871,16 +872,15 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         if not self._mask:
             return
 
-        if (direction == gtk.DIR_TAB_FORWARD or
-            direction == gtk.DIR_DOWN):
-            inc = 1
-        if (direction == gtk.DIR_TAB_BACKWARD or
-            direction == gtk.DIR_UP):
-            inc = -1
-
         field = self._current_field
 
-        field += inc
+        if (direction == gtk.DIR_TAB_FORWARD or
+            direction == gtk.DIR_DOWN):
+            field += 1
+        elif (direction == gtk.DIR_TAB_BACKWARD or
+              direction == gtk.DIR_UP):
+            field = -1
+
         # Leaving the entry
         if field == len(self._mask_fields) or field == -1:
             self.select_region(0, 0)
@@ -933,7 +933,8 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         if field is None:
             self.set_position(self._pos)
         else:
-            self._current_field = field
+            if self._current_field != -1:
+                self._current_field = field
             self._pos = pos
 
     def _on_changed(self, widget):
@@ -991,7 +992,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         else:
             mode = self._mode
 
-        values = {}
+        values = set()
         if mode == ENTRY_MODE_TEXT:
             if sort:
                 itemdata.sort()
@@ -1001,7 +1002,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
                     raise KeyError("Tried to insert duplicate value "
                                    "%r into the entry" % item)
                 else:
-                    values[item] = None
+                    values.add(item)
 
                 model.append((item, None))
         elif mode == ENTRY_MODE_DATA:
@@ -1010,11 +1011,14 @@ class KiwiEntry(PropertyObject, gtk.Entry):
 
             for item in itemdata:
                 text, data = item
-                if text in values:
-                    raise KeyError("Tried to insert duplicate value "
-                                   "%r into the entry" % text)
-                else:
-                    values[text] = None
+                # Add (n) to the end in case of duplicates
+                count = 1
+                orig = text
+                while text in values:
+                    text = orig + ' (%d)' % count
+                    count += 1
+
+                values.add(text)
                 model.append((text, data))
         else:
             raise TypeError("Incorrect format for itemdata; see "
