@@ -42,11 +42,12 @@ from gtk import gdk
 from kiwi.environ import environ
 from kiwi.interfaces import IValidatableProxyWidget
 from kiwi.log import Logger
+from kiwi.python import namedAny
 from kiwi.utils import gsignal, type_register
 from kiwi.ui.gadgets import quit_if_last
 from kiwi.ui.proxy import Proxy
 
-log = Logger('view')
+log = Logger('kiwi.view')
 
 _non_interactive = (
     gtk.Label,
@@ -112,7 +113,7 @@ class SignalBroker(object):
             match = method_regex.match(fname)
             if match is None:
                 continue
-            after, w_name, signal = match.groups()
+            on_after, w_name, signal = match.groups()
             widget = getattr(view, w_name, None)
             if widget is None:
                 raise AttributeError("couldn't find widget %s in %s"
@@ -124,11 +125,13 @@ class SignalBroker(object):
             # Must use getattr; using the class method ends up with it
             # being called unbound and lacking, thus, "self".
             try:
-                if after:
+                if on_after == 'on':
+                    signal_id = widget.connect(signal, methods[fname])
+                elif on_after == 'after':
                     signal_id = widget.connect_after(signal, methods[fname])
                 else:
-                    signal_id = widget.connect(signal, methods[fname])
-            except TypeError:
+                    raise AssertionError
+            except TypeError, e:
                 raise TypeError("Widget %s doesn't provide a signal %s" % (
                                 widget.__class__, signal))
             self._autoconnected.setdefault(widget, []).append((
@@ -603,7 +606,7 @@ class SlaveView(gobject.GObject):
 
     def get_sizegroups(self):
         """
-        @returns: a list of sizegroups for the current view.
+        Get a list of sizegroups for the current view.
         """
         if not self._glade_adaptor:
             return []
@@ -957,6 +960,13 @@ def _get_libglade():
         return
     return LibgladeWidgetTree
 
+def _get_gaxml():
+    try:
+        from kiwi.ui.gaxmlloader import GAXMLWidgetTree
+    except ImportError:
+        return
+    return GAXMLWidgetTree
+
 def _open_glade(view, gladefile, domain):
     if not gladefile:
         raise ValueError("A gladefile wasn't provided.")
@@ -964,8 +974,12 @@ def _open_glade(view, gladefile, domain):
         raise TypeError(
               "gladefile should be a string, found %s" % type(gladefile))
 
-    filename = os.path.splitext(os.path.basename(gladefile))[0]
-    gladefile = environ.find_resource("glade", filename + '.glade')
+    if gladefile.endswith('.ui'):
+        directory = os.path.dirname(namedAny(view.__module__).__file__)
+        gladefile = os.path.join(directory, gladefile)
+    else:
+        filename = os.path.splitext(os.path.basename(gladefile))[0]
+        gladefile = environ.find_resource("glade", filename + '.glade')
 
     fp = open(gladefile)
     sniff = fp.read(200)
@@ -981,6 +995,9 @@ def _open_glade(view, gladefile, domain):
     if 'glade-2.0.dtd' in sniff:
         WidgetTree = _get_libglade()
         loader_name = 'libglade'
+    elif 'gaxml-0.1.dtd' in sniff:
+        WidgetTree = _get_gaxml()
+        loader_name = 'gaxml'
     else:
         # gazpacho:
         #<?xml version="1.0" standalone="no"?> <!--*- mode: xml -*-->
