@@ -27,8 +27,12 @@
 The L{Label} is also extended to support some basic markup like
 L{Label.set_bold}"""
 
+import datetime
+
 import gtk
 
+from kiwi.datatypes import number, ValueUnset, converter
+from kiwi.enums import Alignment
 from kiwi.python import deprecationwarn
 from kiwi.ui.gadgets import set_foreground
 from kiwi.ui.proxywidget import ProxyWidgetMixin
@@ -36,8 +40,13 @@ from kiwi.utils import PropertyObject, type_register
 
 class ProxyLabel(PropertyObject, gtk.Label, ProxyWidgetMixin):
     __gtype_name__ = 'ProxyLabel'
+    allowed_data_types = (basestring, datetime.date, datetime.datetime,
+                          datetime.time) + number
+    _label_replacements = {}
+
     def __init__(self, label='', data_type=None):
         """
+        Create a new ProxyLabel object.
         @param label: initial text
         @param data_type: data type of label
         """
@@ -55,8 +64,37 @@ class ProxyLabel(PropertyObject, gtk.Label, ProxyWidgetMixin):
                            'xx-large')
 
         self.connect("notify::label", self._on_label_changed)
+        self._block_notify_label = False
+
+    def prop_set_data_type(self, data_type):
+        data_type = super(ProxyLabel, self).prop_set_data_type(data_type)
+        if not data_type:
+            return
+
+        conv = converter.get_converter(data_type)
+        if conv.align == Alignment.RIGHT:
+            self.set_property('xalign', 1.0)
+
+        return data_type
+
+    #@classmethod
+    def replace(cls, markup, value):
+        cls._label_replacements[markup] = value
+    replace = classmethod(replace)
 
     def _on_label_changed(self, label, param):
+        if self._block_notify_label:
+            self._block_notify_label = False
+            return
+
+        text = self.get_property('label')
+        for rep in self._label_replacements.keys():
+            if rep in text:
+                text = text.replace(rep, self._label_replacements[rep])
+
+        self._block_notify_label = True
+        self.set_property('label', text)
+
         # Since most of the time labels do not have a model attached to it
         # we should just emit a signal if a model is defined
         if self.model_attribute:
@@ -66,7 +104,7 @@ class ProxyLabel(PropertyObject, gtk.Label, ProxyWidgetMixin):
         return self._from_string(self.get_text())
 
     def update(self, data):
-        if data is None:
+        if data is None or data is ValueUnset:
             text = ""
         else:
             text = self._as_string(data)
