@@ -29,6 +29,7 @@ import datetime
 
 import gtk
 
+from kiwi import ValueUnset
 from kiwi.datatypes import number
 from kiwi.python import deprecationwarn
 from kiwi.ui.proxywidget import ValidatableProxyWidgetMixin
@@ -38,6 +39,7 @@ class ProxyTextView(PropertyObject, gtk.TextView, ValidatableProxyWidgetMixin):
     __gtype_name__ = 'ProxyTextView'
     allowed_data_types = (basestring, datetime.date) + number
     def __init__(self):
+        self._is_unset = True
         gtk.TextView.__init__(self)
         PropertyObject.__init__(self, data_type=str)
         ValidatableProxyWidgetMixin.__init__(self)
@@ -48,19 +50,27 @@ class ProxyTextView(PropertyObject, gtk.TextView, ValidatableProxyWidgetMixin):
         self.set_buffer(self._textbuffer)
 
     def _on_textbuffer__changed(self, textbuffer):
+        self._is_unset = False
         self.emit('content-changed')
         self.read()
 
     def read(self):
+        if self._is_unset:
+            return ValueUnset
         textbuffer = self._textbuffer
         data = textbuffer.get_text(textbuffer.get_start_iter(),
                                    textbuffer.get_end_iter())
         return self._from_string(data)
 
     def update(self, data):
-        if data is None:
+        if data is ValueUnset:
+            self._textbuffer.set_text("")
+            self._is_unset = True
+            return
+        elif data is None:
             text = ""
         else:
+            self.is_unset = False
             text = self._as_string(data)
 
         self._textbuffer.set_text(text)
