@@ -2,7 +2,7 @@
 import unittest
 import gtk
 
-from kiwi.ui.delegates import Delegate
+from kiwi.ui.delegates import Delegate, GladeDelegate
 from utils import refresh_gui
 
 class A:
@@ -62,11 +62,10 @@ class ClickCounter(Delegate):
     def on_button__clicked(self, *args):
         self.clicks += 1
 
-class GladeClickCounter(Delegate):
+class GladeClickCounter(GladeDelegate):
     def __init__(self):
-        Delegate.__init__(self, gladefile="simple_button",
-                          widgets=['button'],
-                          delete_handler=self.quit_if_last)
+        GladeDelegate.__init__(self, gladefile="simple_button",
+                               delete_handler=self.quit_if_last)
 
         self.clicks = 0
 
