@@ -13,6 +13,7 @@ from kiwi.ui.widgets.checkbutton import ProxyCheckButton
 from kiwi.ui.widgets.entry import ProxyEntry
 from kiwi.ui.widgets.label import ProxyLabel
 from kiwi.ui.widgets.radiobutton import ProxyRadioButton
+from kiwi.ui.widgets.scale import ProxyHScale, ProxyVScale
 from kiwi.ui.widgets.spinbutton import ProxySpinButton
 from kiwi.ui.widgets.textview import ProxyTextView
 from kiwi.ui.widgets.combo import ProxyComboEntry, ProxyComboBox
@@ -45,6 +46,8 @@ class Model(Settable):
                           radiobutton='first',
                           label='label',
                           spinbutton=100,
+                          hscale=100.0,
+                          vscale=100.0,
                           textview='sliff',
                           comboentry='CE1',
                           combobox='CB1',
@@ -57,6 +60,8 @@ class TestProxy(unittest.TestCase):
         self.view.add('entry', str, ProxyEntry)
         self.view.add('label', str, ProxyLabel)
         self.view.add('spinbutton', int, ProxySpinButton)
+        self.view.add('hscale', float, ProxyHScale)
+        self.view.add('vscale', float, ProxyVScale)
         self.view.add('button', str, ProxyButton)
         self.view.add('buttonpixbuf', gdk.Pixbuf, ProxyButton)
 
@@ -67,6 +72,9 @@ class TestProxy(unittest.TestCase):
         self.radio_second.set_group(self.radio_first)
         self.radio_second.set_property('data_value', 'second')
 
+        self.view.hscale.get_adjustment().upper = 200
+        self.view.vscale.get_adjustment().upper = 250
+
         self.comboentry = self.view.add('comboentry', str, ProxyComboEntry)
         self.comboentry.prefill(['CE1','CE2','CE3'])
         self.comboentry.show()
@@ -100,6 +108,16 @@ class TestProxy(unittest.TestCase):
         self.radio_first.set_active(True)
         self.assertEqual(self.model.radiobutton, 'first')
 
+    def testHScale(self):
+        self.assertEqual(self.model.vscale, 100)
+        self.view.vscale.set_value(220)
+        self.assertEqual(self.model.vscale, 220)
+
+    def testVScale(self):
+        self.assertEqual(self.model.vscale, 100)
+        self.view.vscale.set_value(200)
+        self.assertEqual(self.model.vscale, 200)
+
     def testSpinButton(self):
         self.assertEqual(self.model.spinbutton, 100)
         self.view.spinbutton.set_text("200")
@@ -148,14 +166,15 @@ class TestProxy(unittest.TestCase):
         self.radio_second.set_active(True)
 
         self.proxy.set_model(None)
-        self.assertEqual(self.view.entry.read(), '')
+
+        self.assertEqual(self.view.entry.read(), ValueUnset)
         self.assertEqual(self.view.checkbutton.read(), False)
         self.assertEqual(self.view.radiobutton.read(), 'first')
-        self.assertEqual(self.view.label.read(), '')
-        self.assertEqual(self.view.spinbutton.read(), ValueUnset)
-        self.assertEqual(self.view.textview.read(), '')
-        self.assertEqual(self.view.comboentry.read(), None)
-        self.assertEqual(self.view.combobox.read(), 'CB1')
+        self.assertEqual(self.view.label.read(), '', 'label')
+        self.assertEqual(self.view.spinbutton.read(), ValueUnset, 'spinbutton')
+        self.assertEqual(self.view.textview.read(), ValueUnset, 'textview')
+        self.assertEqual(self.view.comboentry.read(), None, 'comboentry')
+        self.assertEqual(self.view.combobox.read(), 'CB1', 'combobox')
 
     def testValueUnset(self):
         self.view.entry.update(ValueUnset)
