@@ -1,8 +1,7 @@
 import os
+import subprocess
 import sys
-import traceback
 import unittest
-import popen2
 
 def setup(self, rootdir):
     self._dir = os.getcwd()
@@ -12,14 +11,11 @@ def teardown(self):
     os.chdir(self._dir)
 
 def test_filename(rootdir, filename):
-    cmd = '%s %s %s' % (sys.executable,
-                        os.path.join(rootdir, 'bin', 'kiwi-ui-test'),
-                        os.path.join('tests', 'ui', filename))
-    if sys.platform == 'win32':
-        status = os.system(cmd)
-    else:
-        p = popen2.Popen3(cmd)
-        status = os.waitpid(p.pid, 0)[1]
+    cmd = '%s %s -v %s' % (sys.executable,
+                           os.path.join(rootdir, 'bin', 'kiwi-ui-test'),
+                           os.path.join('tests', 'ui', filename))
+    proc = subprocess.Popen(cmd, shell=True)
+    status = proc.wait()
 
     if status != 0:
         raise AssertionError("UI Test %s failed" % filename)
@@ -28,15 +24,14 @@ def create():
     testdir = os.path.dirname(__file__)
     uidir = os.path.join(testdir, 'ui')
     rootdir = os.path.dirname(testdir)
-
     tests = {}
     tests['setUp'] = lambda self, rootdir=rootdir: setup(self, rootdir)
     tests['tearDown'] = lambda self, rootdir=rootdir: teardown(self)
 
     for filename in os.listdir(uidir):
-        if not filename.endswith('.py'):
+        if not filename.endswith('.doctest'):
             continue
-        name = 'test_' + filename[:-3]
+        name = 'test_' + filename[:-7]
 
         full = os.path.join(uidir, filename)
 
@@ -46,7 +41,6 @@ def create():
         except TypeError:
             pass
         tests[name] = func
-
     return type('TestUI', (unittest.TestCase,), tests)
 
 # Disable UI tests on win32, they do not quite work yet.
